VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 1  'NoTransaction
END
Attribute VB_Name = "Bank"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

' Class module in data tier

Const MODULE_NAME As String = "Bank_DB.Bank"

' NOTE: THESE VALUES CORRESPOND
' TO ENTRIES IN THE TxType TABLE
' THIS ENUM AND THE TxType TABLE
' MUST BE KEPT SYNCHRONIZED!!!
Public Enum ETransactionType
  txTransfer = 1
  txBillPayment = 2
End Enum

Private m_objDBHelper As DBHelper

Private Sub Class_Initialize()
' Initialize the helper object to run stored procedures
   Set m_objDBHelper = CreateObject("Bank_DB.DBHelper")
End Sub

Private Sub Class_Terminate()
' Destroy the helper object
   Set m_objDBHelper = Nothing
End Sub

Public Function VerifyLogin(ByVal BankID As Integer, _
  ByVal CardID As Long, ByVal Password As String, _
  ByRef InvalidCount As Variant) As Boolean
' Verify the WROBA login

  Dim strUserInfo As String
  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  Dim cmd As ADODB.Command
  
  On Error GoTo ErrorHandler

  ' Pessimistic assumption of login failure
  VerifyLogin = False
  
  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' The following code directly runs the stored procedure.
  ' Set up Command and Connection objects
  Set cmd = CtxCreateObject("ADODB.Command")
  
  'Run the procedure
  cmd.ActiveConnection = strDSN
  cmd.CommandText = "sp_select_WROBACard_PwdCnt"
  cmd.CommandType = adCmdStoredProc
  
  ' Set up the parameters for the procedure call
  With cmd
    .Parameters.Append .CreateParameter(, adInteger, _
      adParamReturnValue)
    .Parameters.Append .CreateParameter("@CardId_1", _
      adInteger, adParamInput, 4, CardID)
    .Parameters.Append .CreateParameter("@Password_2", _
      adChar, adParamInput, 10, Password)
    .Parameters.Append .CreateParameter("@InvalidPasswordCnt_3", _
      adInteger, adParamOutput, 4)
  End With
  
  ' Execute the stored procedure directly
  cmd.Execute , , ADODB.adExecuteNoRecords
  
  If cmd.Parameters(0).Value <> 0 Then
    InvalidCount = cmd.Parameters(3).Value
    VerifyLogin = True
  End If
   
Clean_Up:
  Set objBankLocator = Nothing
  Set cmd.ActiveConnection = Nothing
  Set cmd = Nothing
  CtxSetComplete
   
  Exit Function

ErrorHandler:
  RaiseError MODULE_NAME, "VerifyLogin"
  GoTo Clean_Up
End Function

Public Property Get ChangeLogin(ByVal BankID As Integer, _
  ByVal CardID As Long, ByVal Password As String) As Boolean
' Verify the WROBA login

  Dim strUserInfo As String
  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  Dim cmd As ADODB.Command
  
  On Error GoTo ErrorHandler

  ' Pessimistic assumption of failure
  ChangeLogin = False
  
  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' The following code directly runs the stored procedure.
  ' Set up Command and Connection objects
  Set cmd = CtxCreateObject("ADODB.Command")
  
  'Run the procedure
  cmd.ActiveConnection = strDSN
  cmd.CommandText = "sp_update_WROBACard_Pwd"
  cmd.CommandType = adCmdStoredProc
  
  ' Set up the parameters for the procedure call
  With cmd
    .Parameters.Append .CreateParameter(, adInteger, _
      adParamReturnValue)
    .Parameters.Append .CreateParameter("@CardId_1", _
      adInteger, adParamInput, 4, CardID)
    .Parameters.Append .CreateParameter("@Password_2", _
      adChar, adParamInput, 10, Password)
  End With

  ' Execute the stored procedure directly
  cmd.Execute , , ADODB.adExecuteNoRecords
  
  If cmd.Parameters(0).Value = 0 Then
    ChangeLogin = False
  Else
    ChangeLogin = True
  End If
   
Clean_Up:
  Set objBankLocator = Nothing
  Set cmd.ActiveConnection = Nothing
  Set cmd = Nothing
  CtxSetComplete
  
  Exit Property

ErrorHandler:
  RaiseError MODULE_NAME, "ChangeLogin"
  GoTo Clean_Up
End Property

Public Sub UpdatePasswordCount(ByVal BankID As Integer, _
  ByVal CardID As Long, ByVal InvalidCount As Integer)

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure
  m_objDBHelper.RunSP strDSN, "sp_update_WROBACard_PwdCnt", _
    Array("@CardId_1", adInteger, adSizeInteger, CardID), _
    Array("@InvalidPasswordCnt_2", adTinyInt, adSizeTinyInt, _
    InvalidCount)
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Sub

ErrorHandler:
  RaiseError MODULE_NAME, "UpdatePasswordCount"
  GoTo Clean_Up
End Sub

Public Sub WROBALock(ByVal BankID As Integer, _
  ByVal CardID As Long)

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure
  m_objDBHelper.RunSP strDSN, "sp_insert_TxLock", _
    Array("@BankId_1", adSmallInt, adSizeSmallInt, BankID), _
    Array("@CardId_2", adInteger, adSizeInteger, CardID)
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Sub

ErrorHandler:
  Debug.Print Err.Number
  RaiseError MODULE_NAME, "WROBALock"
  GoTo Clean_Up
End Sub

Public Sub WROBAUnLock(ByVal BankID As Integer, _
  ByVal CardID As Long)

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure
  m_objDBHelper.RunSP strDSN, "sp_delete_TxLock", _
    Array("@BankId_1", adSmallInt, adSizeSmallInt, BankID), _
    Array("@CardId_2", adInteger, adSizeInteger, CardID)
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Sub

ErrorHandler:
  RaiseError MODULE_NAME, "WROBAUnLock"
  GoTo Clean_Up
End Sub

Public Property Get GetAccounts(ByVal BankID As Integer, _
  ByVal CardID As Long) As ADODB.Recordset

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure
  Set GetAccounts = m_objDBHelper.RunSPReturnRS(strDSN, _
    "sp_select_BankAccount_By_CardId", _
    Array("@CardId_1", adInteger, adSizeInteger, CardID))
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Set GetAccounts = Nothing
  RaiseError MODULE_NAME, "GetAccounts"
  GoTo Clean_Up
End Property

Public Sub GetAccountBalance(ByVal BankID As Integer, _
  ByVal AccountID As Long, ByRef AccountTypeDesc As Variant, _
  ByRef Balance As Variant, ByRef DailyRemaining As Variant, _
  ByRef OverdraftLimit As Variant)

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  Dim cmd As ADODB.Command
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' The following code directly runs the stored procedure.
  ' Set up Command and Connection objects
  Set cmd = New ADODB.Command

  ' Init the ADO objects & the stored proc parameters
  cmd.ActiveConnection = strDSN
  cmd.CommandText = "sp_select_BankAccount_By_AccountId"
  cmd.CommandType = adCmdStoredProc
  
  ' Add input parameters
  cmd.Parameters.Append cmd.CreateParameter("@AccountId_1", _
    adInteger, adParamInput, adSizeInteger, AccountID)
  
  ' Add output parameters
  cmd.Parameters.Append cmd.CreateParameter("@AccountTypeDesc_2", _
    adChar, adParamOutput, 15)
  cmd.Parameters.Append cmd.CreateParameter("@Balance_3", _
    adCurrency, adParamOutput)
  cmd.Parameters.Append cmd.CreateParameter("@DailyRemaining_4", _
    adCurrency, adParamOutput)
  cmd.Parameters.Append cmd.CreateParameter("@OverdraftLimit_5", _
    adCurrency, adParamOutput)
  
  ' Execute directly without a resulting recordset and
  ' pull out the return parameters
  cmd.Execute , , adExecuteNoRecords
  
  AccountTypeDesc = cmd.Parameters("@AccountTypeDesc_2").Value
  Balance = cmd.Parameters("@Balance_3").Value
  DailyRemaining = cmd.Parameters("@DailyRemaining_4").Value
  OverdraftLimit = cmd.Parameters("@OverdraftLimit_5").Value
  
  ' Disconnect the recordset, and clean up
Clean_Up:
  Set cmd.ActiveConnection = Nothing
  Set cmd = Nothing
  
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Sub

ErrorHandler:
  RaiseError MODULE_NAME, "GetAccountBalance"
  GoTo Clean_Up
End Sub

Public Property Get GetAccountHistory(ByVal BankID As Integer, _
  ByVal AccountID As Long, ByVal MaxTransactionDate As Date) _
  As ADODB.Recordset

  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure
  Set GetAccountHistory = m_objDBHelper.RunSPReturnRS(strDSN, _
    "sp_select_AccountHistory_By_AccountId", _
    Array("@AccountId_1", adInteger, adSizeInteger, AccountID), _
    Array("@MaxTxDate_2", adDBTimeStamp, adSizeDateTime, _
    MaxTransactionDate))
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Set GetAccountHistory = Nothing
  RaiseError MODULE_NAME, "GetAccountHistory"
  GoTo Clean_Up
End Property

Public Property Get GetBanks(ByVal BankID As Integer) As ADODB.Recordset
   
  Dim objBankLocator As Bank_DB.BankLocator
  Dim strDSN As String
  
  On Error GoTo ErrorHandler

  ' Locate the selected bank and get the DSN for it
  Set objBankLocator = New Bank_DB.BankLocator
  strDSN = objBankLocator.GetBank(BankID)
  
  ' Run the stored procedure
  Set GetBanks = m_objDBHelper.RunSPReturnRS(strDSN, _
    "sp_select_Bank")
  
Clean_Up:
  Set objBankLocator = Nothing
  CtxSetComplete
  Exit Property

ErrorHandler:
  Set GetBanks = Nothing
  RaiseError MODULE_NAME, "GetBanks"
  GoTo Clean_Up
End Property
