VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "FutureValueOfAnnuity"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private m_objInvestmentDetails As PeriodicInvestments
Private m_curInvestmentAmount As Currency
Private m_dblRate As Double
Private m_lngNumInvestments As Long
Private m_lngInvestmentsPerYear As Long
Private m_curExtraInvestment As Currency
Private m_curDesiredAmount As Currency
Private m_curTotalEarnings As Currency
Private m_curTotalInvestments As Currency

Private Const INVALIDAMOUNTERR As Long = vbObjectError + 1001
Private Const INVALIDAMOUNT As String = _
    "The amount of the loan must be specified"
Private Const INVALIDRATEERR As Long = vbObjectError + 1002
Private Const INVALIDRATE As String = _
    "The specified interest rate is invalid"
Private Const INVALIDNUMINVESTSERR As Long = vbObjectError + 1003
Private Const INVALIDNUMINVESTS As String = _
    "The number of payments must be greater than 0"
Private Const INVALIDINVESTSPERERR As Long = vbObjectError + 1004
Private Const INVALIDINVESTSPER As String = _
    "There must be at least one payment due each year"
Private Sub Class_Initialize()
   Set m_objInvestmentDetails = New PeriodicInvestments
End Sub

Private Sub Class_Terminate()
    ' ensure that the memory for the objects is freed
    Set m_objInvestmentDetails = Nothing
End Sub

' Create the properties for our class

' the DesiredAmount property indicates the target amount to accumulate
Public Property Get DesiredAmount() As Currency
    DesiredAmount = m_curDesiredAmount
End Property

Public Property Let DesiredAmount(curAmount As Currency)
    ' check for valid property value
    If curAmount <= 0 Then
        Err.Raise INVALIDAMOUNTERR, _
    "FinancialUtility:Amortization:Amount", INVALIDAMOUNT
    End If
    m_curDesiredAmount = curAmount
End Property

' the Rate property is the interest rate for the investment
' it is expressed in percentages - for example 6.5 is 6.5% or .065 per year
Public Property Get Rate() As Double
    Rate = m_dblRate
End Property

Public Property Let Rate(dblRate As Double)
    ' check for valid property value
    If dblRate <= 0 Or dblRate > 50 Then
        Err.Raise INVALIDRATEERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDRATE
    End If
    m_dblRate = dblRate
End Property

' the NumberOfPayouts property identifies the number of
' payouts that are scheduled to be made over the life of
' the annuity
Public Property Get NumberOfInvestments() As Long
    NumberOfInvestments = m_lngNumInvestments
End Property

Public Property Let NumberOfInvestments(lngInvestments As Long)
    ' check for valid property value
    If lngInvestments <= 0 Then
        Err.Raise INVALIDNUMINVESTSERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDNUMINVESTS
    End If
    m_lngNumInvestments = lngInvestments
End Property

' the InvestmentsPerYear property identifies the number of investments
' that are required each year - used to calculate the periodic rate
Public Property Get InvestmentsPerYear() As Long
    InvestmentsPerYear = m_lngInvestmentsPerYear
End Property

Public Property Let InvestmentsPerYear(lngInvestmentsPerYear As Long)
    ' check for valid property value
    If lngInvestmentsPerYear <= 0 Then
        Err.Raise INVALIDINVESTSPERERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDINVESTSPER
    End If
    m_lngInvestmentsPerYear = lngInvestmentsPerYear
End Property

' the InvestmentAmount property is set when the calculate method
' is called - this is a read only property that tells the
' required investment for the annuity
Public Property Get InvestmentAmount() As Currency
    InvestmentAmount = m_curInvestmentAmount
End Property

' the TotalInvestments property is set when the calculate method
' is called - this is a read only property that tells the
' total amount of periodic investments over the life of the annuity
Public Property Get TotalInvestments() As Currency
    TotalInvestments = m_curTotalInvestments
End Property

' the TotalEarnings property is set when the calculate method
' is called - this is a read only property that tells the
' total amount of earnings over the life of the annuity
Public Property Get TotalEarnings() As Currency
    TotalEarnings = m_curTotalEarnings
End Property

' the InvestmentDetails property returns a reference to a
' PeriodicInvestments object that holds information about
' each scheduled investment for the annuity
Public Property Get InvestmentDetails() As PeriodicInvestments
    Set InvestmentDetails = m_objInvestmentDetails
End Property

' Create the methods for our class

' the calculate method is the only method for our class
' but it is the work horse where all of the information is
' generated
' it accepts optional parameters and sets the properties
' accordingly if they are passed in
Public Sub Calculate(Optional curDesiredAmount As Currency, _
Optional dblRate As Double, Optional lngNumInvestments As Long, _
Optional lngInvestmentsPerYear As Long)

    ' declare local variables
    Dim dblPeriodicRate As Double
    Dim dblFactor1 As Double
    Dim dblFactor3 As Double
    Dim curCurrentBalance As Currency
    Dim lngCounter As Long
    Dim objInvestment As PeriodicInvestment

    On Error GoTo CALCULATE_ERROR

    ' check to see if we need to set
    ' the properties to the input values
    If curDesiredAmount <> 0 Then m_curDesiredAmount = _
    curDesiredAmount
    If dblRate <> 0 Then m_dblRate = dblRate
    If lngNumInvestments <> 0 Then m_lngNumInvestments = _
    lngNumInvestments
    If lngInvestmentsPerYear <> 0 Then m_lngInvestmentsPerYear = _
    lngInvestmentsPerYear

    ' ensure that we have the required information or send an
    ' error to the user
    If m_curDesiredAmount <= 0 Then
        Err.Raise INVALIDAMOUNTERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDAMOUNT
    End If
    
    If m_dblRate <= 0 Or dblRate > 50 Then
        Err.Raise INVALIDRATEERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDRATE
    End If
    
    If m_lngNumInvestments <= 0 Then
        Err.Raise INVALIDNUMINVESTSERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDNUMINVESTS
    End If
    
    If m_lngInvestmentsPerYear <= 0 Then
        Err.Raise INVALIDINVESTSPERERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDINVESTSPER
    End If
    
    ' calculate the periodic interest rate based on the rate
    ' and number of payouts per year
    dblPeriodicRate = (m_dblRate / 100 / m_lngInvestmentsPerYear)

    'Calculate Periodic Payout
    ' use the periodic rate and other inputs to calculate
    ' the required payment.  This is broken into steps
    ' to facilitate better understanding
    dblFactor1 = (1 + dblPeriodicRate) ^ m_lngNumInvestments
    dblFactor3 = (dblFactor1 - 1) / dblPeriodicRate
    m_curInvestmentAmount = m_curDesiredAmount / dblFactor3

    ' compute each periodic investment for the annuity
    ' keep track of the current balance so we can
    ' identify each investments details
    curCurrentBalance = 0

    'Calculate TotalEarnings and Details for each investment
    For lngCounter = 1 To m_lngNumInvestments
        ' for each investment create a PeriodicInvestment object
        ' and set its properties
        Set objInvestment = New PeriodicInvestment

        With objInvestment
            .InvestmentNumber = lngCounter
            .InvestmentAmount = m_curInvestmentAmount
            .EarningsForPeriod = curCurrentBalance * dblPeriodicRate
            ' keep a running total of overall Earnings
            m_curTotalEarnings = m_curTotalEarnings + .EarningsForPeriod
            .CurrentBalance = curCurrentBalance + .EarningsForPeriod + _
        .InvestmentAmount

            curCurrentBalance = .CurrentBalance
        End With

        ' add the new payout to the Payouts collection
        m_objInvestmentDetails.Add objInvestment
        ' free our object - VB does not always work correctly if we don't
        Set objInvestment = Nothing
    Next

    ' set the TotalPayouts property
    m_curTotalInvestments = m_curInvestmentAmount * m_lngNumInvestments

    Exit Sub

CALCULATE_ERROR:
    ' if required add specific error handling here
    ' for now we will just raise all errors to the calling
    ' program
    Err.Raise Err.Number

End Sub

' Create supporting procedures for our class
' N/A


