VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "SalesOrder"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Ext_KEY = "RVB_UniqueId" ,"37487CB0033C"
'This is the UI centric SalesOrder class.  It uses the data centric SalesOrderN
'(non-transact) and SalesOrderT (transact) classes to communicate to the
'backend database.

Option Explicit

'##ModelId=374C45C402C2
Public Event Changed()

'##ModelId=374C45C103AE
Private Const mcClassName As String = "UIObjects.SalesOrderItem"
'##ModelId=374C45C2007A
Private Const mcObjectError As String = vbObjectError + &H1120

'##ModelId=374C45C2012F
Private mID As Long
'##ModelId=374C45C201ED
Private mDate As Date

'##ModelId=374882FD0101
Private mCustomer As Customer

'##ModelId=37488E8F00D9
Private mItems As Collection

'##ModelId=3748FCA2031C
Private mSalesOrderN As DCSalesOrder.SalesOrderN

'##ModelId=3748FCC50114
Private mSalesOrderT As DCSalesOrder.SalesOrderT

'The SalesOrder ID.  It is automatically generated by the database and
'read-only to the clients.
'##ModelId=37487E8601B1
Public Property Get ID() As Long
    Let ID = mID
End Property

'Sales order date.  This is assigned when an order is saved to the database and
'is read-only to the clients.
'##ModelId=37487EF400DD
Public Property Get OrderDate() As Date
    Let OrderDate = mDate
End Property

'##ModelId=374C45C202FB
Public Property Let CustomerID(Value As String)
    mCustomer.Read Value
End Property

'Retrieve the Customer object associated with a Sales Order.
'##ModelId=374884F5026E
Public Property Get Customer() As UIObjects.Customer
    Set Customer = mCustomer
End Property

'##ModelId=374C45C30090
Public Property Get Items() As Collection
    Set Items = mItems
End Property

'Retrieve a Sales Order record and populates object properties.
'##ModelId=37490CE10100
Public Sub Read(OrderID As Long)
    On Error GoTo ErrHandler

    Dim rs As ADODB.Recordset

    Set rs = mSalesOrderN.GetByID(OrderID)
    Populate rs

FuncExit:
    If Not rs Is Nothing Then
        If rs.State = adStateOpen Then
            rs.Close
        End If
        Set rs = Nothing
    End If
    Exit Sub

ErrHandler:
    Err.Raise mcObjectError + 1, mcClassName, Err.Description
    Resume FuncExit
End Sub

'Save a Sales Order and its Items.
'##ModelId=37490D500290
Public Sub Save()
    On Error GoTo ErrHandler
    
    Dim OrderData As String
    OrderData = Serialize
    mID = mSalesOrderT.Save(OrderData)
    
    ' Update object properties
    Read ID

FuncExit:
    Exit Sub

ErrHandler:
    Err.Raise mcObjectError + 2, mcClassName, Err.Description
    Resume FuncExit
End Sub

'Delete a Sales Order and its Items.
'##ModelId=37490D740350
Public Sub Delete()
    On Error GoTo ErrHandler
    mSalesOrderT.Delete ID
    Read 0

FuncExit:
    Exit Sub

ErrHandler:
    Err.Raise mcObjectError + 3, mcClassName, Err.Description
    Resume FuncExit
End Sub

'##ModelId=374C45C3025D
Public Sub AddItem(ProductID As Long, Quantity As Integer)
    On Error GoTo ErrHandler

    Dim OrderItem As SalesOrderItem
    Set OrderItem = New SalesOrderItem
    With OrderItem
        .ProductID = ProductID
        .Quantity = Quantity
    End With
    
    mItems.Add OrderItem
    RaiseEvent Changed
    
FuncExit:
    Set OrderItem = Nothing
    Exit Sub

ErrHandler:
    Err.Raise mcObjectError + 4, mcClassName, Err.Description
    Resume FuncExit
End Sub

'Populate object properties from the recordset retrieved by the Read method.
'##ModelId=37490D8F036D
Private Sub Populate(OrderRS As ADODB.Recordset)
    On Error GoTo ErrHandler

    Dim ItemRS As ADODB.Recordset
    Dim Item As SalesOrderItem
    Dim ItemIndex As Integer

    If Not (OrderRS.BOF And OrderRS.EOF) Then
        With OrderRS
            mID = .Fields("OrderID").Value
            CustomerID = .Fields("CustomerID").Value
            mDate = .Fields("OrderDate").Value

            Set ItemRS = .Fields("rsItems").Value
        End With
        
        ' Clear the Items collection
        For ItemIndex = 1 To Items.Count
            mItems.Remove 1
        Next ItemIndex
        
        With ItemRS
            While Not .EOF
                Set Item = New SalesOrderItem
                Item.ProductID = .Fields("ProductID").Value
                Item.Quantity = .Fields("Quantity")
                mItems.Add Item
                .MoveNext
            Wend
        End With
    Else
        mID = 0
        Customer.Clear
        mDate = Now
        For ItemIndex = 1 To mItems.Count
            mItems.Remove 1
        Next ItemIndex
    End If
        
    RaiseEvent Changed

FuncExit:
    Exit Sub

ErrHandler:
    Err.Raise mcObjectError + 11, mcClassName, Err.Description
    Resume FuncExit
End Sub

'Serialize a Sales Order data.
'##ModelId=37490DD102E5
Private Function Serialize() As String
    On Error GoTo ErrHandler
    
    Dim PropBag As PropertyBag
    Dim ItemIndex As Integer
    Dim OrderItem As SalesOrderItem
    
    Set PropBag = New PropertyBag
    With PropBag
        .WriteProperty "CustomerID", Customer.ID
        
        .WriteProperty "ItemCount", mItems.Count
        For ItemIndex = 1 To mItems.Count
            Set OrderItem = mItems.Item(ItemIndex)
            .WriteProperty "Item" & CStr(ItemIndex), OrderItem.Serialize
        Next ItemIndex
        
        Serialize = .Contents
    End With
    
FuncExit:
    On Error Resume Next
    If Not PropBag Is Nothing Then
        Set PropBag = Nothing
    End If
    
    Exit Function
    
ErrHandler:
    Err.Raise mcObjectError + 12, mcClassName, Err.Description
    Resume FuncExit
End Function

Private Sub Class_Initialize()
    On Error GoTo ErrHandler

    Set mSalesOrderN = CreateObject("DCSalesOrder.SalesOrderN")
    Set mSalesOrderT = CreateObject("DCSalesOrder.SalesOrderT")
    Set mItems = New Collection
    Set mCustomer = New UIObjects.Customer

FuncExit:
    Exit Sub

ErrHandler:
    Err.Raise mcObjectError, mcClassName, Err.Description
    Resume FuncExit
End Sub

'##ModelId=374C45C4022C
Private Sub Class_Terminate()
    On Error Resume Next
    Set mSalesOrderN = Nothing
    Set mSalesOrderT = Nothing
    Set mItems = Nothing
    Set mCustomer = Nothing
End Sub

